/// <reference path="../../@mindfusion/drawing/drawing.d.ts" />
/// <reference path="../../@mindfusion/controls/controls.d.ts" />
/// <reference path="../../@mindfusion/diagramming/diagramming.d.ts" />
// Type definitions for MindFusion.Diagramming for React
// Project: https://www.mindfusion.eu/javascript-diagram.html
// Definitions by: MindFusion <https://www.mindfusion.eu>

// Copyright (c) 2020-2022, MindFusion LLC - Bulgaria.

import * as React from "react";
import * as Controls from '@mindfusion/controls';
import * as Drawing from '@mindfusion/drawing';
import * as Diagramming from '@mindfusion/diagramming';

declare module "@mindfusion/diagramming-react-cc"
{
	export class DiagramView extends React.Component<DiagramViewProps, any> { }
	export interface DiagramViewProps {
		id: String;
		diagram: Diagramming.Diagram;
		allowInplaceEdit?: boolean;
		autoResize?: Diagramming.AutoResize;
		backBrush?: any;
		behavior?: Diagramming.Behavior;
		defaultShape?: Diagramming.Shape | string;
		enabled?: boolean;
		linkHeadShapeSize?: number;
		linkShape?: Diagramming.LinkShape;
		roundedLinks?: boolean;
		routeLinks?: boolean;
		showAnchors?: Diagramming.ShowAnchors;
		showGrid?: boolean;
		zoomFactor?: number,
		multiTouchZoom?: boolean;
		multiTouchModify?: boolean;
		enableMultiTouch?: boolean;
		style?: Object,
		onDiagramChanged?: (
			sender: DiagramView,
			args: Controls.EventArgs
		) => void;
		onInitializeLink?: (
			sender: Diagramming.Diagram,
			args: Diagramming.LinkEventArgs
		) => void;
		onInitializeNode?: (
			sender: Diagramming.Diagram,
			args: Diagramming.NodeEventArgs
		) => void;
		onLinkClicked?: (
			sender: Diagramming.Diagram,
			args: Diagramming.LinkEventArgs
		) => void;
		onLinkCreated?: (
			sender: Diagramming.Diagram,
			args: Diagramming.LinkEventArgs
		) => void;
		onLinkDeleted?: (
			sender: Diagramming.Diagram,
			args: Diagramming.LinkEventArgs
		) => void;
		onLinkDoubleClicked?: (
			sender: Diagramming.Diagram,
			args: Diagramming.LinkEventArgs
		) => void;
		onLinkModified?: (
			sender: Diagramming.Diagram,
			args: Diagramming.LinkEventArgs
		) => void;
		onLinkPointed?: (
			sender: Diagramming.Diagram,
			args: Diagramming.LinkEventArgs
		) => void;
		onLinkTextEdited?: (
			sender: Diagramming.Diagram,
			args: Diagramming.LinkEventArgs
		) => void;
		onNodeClicked?: (
			sender: Diagramming.Diagram,
			args: Diagramming.NodeEventArgs
		) => void;
		onNodeCreated?: (
			sender: Diagramming.Diagram,
			args: Diagramming.NodeEventArgs
		) => void;
		onNodeDeleted?: (
			sender: Diagramming.Diagram,
			args: Diagramming.NodeEventArgs
		) => void;
		onNodeDoubleClicked?: (
			sender: Diagramming.Diagram,
			args: Diagramming.NodeEventArgs
		) => void;
		onNodeModified?: (
			sender: Diagramming.Diagram,
			args: Diagramming.NodeEventArgs
		) => void;
		onNodePointed?: (
			sender: Diagramming.Diagram,
			args: Diagramming.NodeEventArgs
		) => void;
		onNodeTextEdited?: (
			sender: Diagramming.Diagram,
			args: Diagramming.NodeEventArgs
		) => void;
		onLinkCreating?: (
			sender: Diagramming.Diagram,
			args: Diagramming.LinkEventArgs
		) => void;
		onLinkDeleting?: (
			sender: Diagramming.Diagram,
			args: Diagramming.LinkEventArgs
		) => void;
		onLinkModifying?: (
			sender: Diagramming.Diagram,
			args: Diagramming.LinkEventArgs
		) => void;
		onNodeCreating?: (
			sender: Diagramming.Diagram,
			args: Diagramming.NodeEventArgs
		) => void;
		onNodeDeleting?: (
			sender: Diagramming.Diagram,
			args: Diagramming.NodeEventArgs
		) => void;
		onNodeModifying?: (
			sender: Diagramming.Diagram,
			args: Diagramming.NodeEventArgs
		) => void;
		onContainerChildAdded?: (
			sender: Diagramming.Diagram,
			args: Diagramming.NodeEventArgs
		) => void;
		onContainerChildAdding?: (
			sender: Diagramming.Diagram,
			args: Diagramming.NodeEventArgs
		) => void;
		onContainerChildRemoving?: (
			sender: Diagramming.Diagram,
			args: Diagramming.NodeEventArgs
		) => void;
		onContainerChildRemoved?: (
			sender: Diagramming.Diagram,
			args: Diagramming.NodeEventArgs
		) => void;
		onContainerFolded?: (
			sender: Diagramming.Diagram,
			args: Diagramming.NodeEventArgs
		) => void;
		onContainerUnfolded?: (
			sender: Diagramming.Diagram,
			args: Diagramming.NodeEventArgs
		) => void;
		onTreeExpanded?: (
			sender: Diagramming.Diagram,
			args: Diagramming.NodeEventArgs
		) => void;
		onTreeCollapsed?: (
			sender: Diagramming.Diagram,
			args: Diagramming.NodeEventArgs
		) => void;
		onSelectionModifying?: (
			sender: Diagramming.Diagram,
			args: Diagramming.SelectionEventArgs
		) => void;
		onRepaint?: (
			sender: Diagramming.Diagram,
			args: Controls.EventArgs
		) => void;
		onSizeChanged?: (
			sender: Diagramming.Diagram,
			args: Controls.EventArgs
		) => void;
		onNodeSelected?: (
			sender: Diagramming.Diagram,
			args: Diagramming.NodeEventArgs
		) => void;
		onNodeDeselected?: (
			sender: Diagramming.Diagram,
			args: Diagramming.NodeEventArgs
		) => void;
		onLinkSelected?: (
			sender: Diagramming.Diagram,
			args: Diagramming.LinkEventArgs
		) => void;
		onLinkDeselected?: (
			sender: Diagramming.Diagram,
			args: Diagramming.LinkEventArgs
		) => void;
		onControlLoaded?: (
			sender: Diagramming.DiagramView
		) => void;
		onHitTestAdjustmentHandles?: (
			sender: Diagramming.Diagram,
			args: Diagramming.NodeEventArgs | Diagramming.LinkEventArgs
		) => void;
		onDrawAdjustmentHandles?: (
			sender: Diagramming.Diagram,
			args: Diagramming.NodeEventArgs | Diagramming.LinkEventArgs
		) => void;
		onClicked?: (
			sender: Diagramming.Diagram,
			args: Diagramming.DiagramEventArgs
		) => void;
		onEnterInplaceEditMode?: (
			sender: Diagramming.Diagram,
			args: Diagramming.InplaceEditEventArgs
		) => void;
		onLeaveInplaceEditMode?: (
			sender: Diagramming.Diagram,
			args: Diagramming.InplaceEditEventArgs
		) => void;
		onCreateEditControl?: (
			sender: Diagramming.Diagram,
			args: Diagramming.InplaceEditEventArgs
		) => void;
		onCellTextEdited?: (
			sender: Diagramming.Diagram,
			args: Diagramming.CellEventArgs
		) => void;
		onLaneGridCellTextEdited?: (
			sender: Diagramming.Diagram,
			args: Diagramming.CellEventArgs
		) => void;
		onSerializeTag?: (
			sender: Diagramming.Diagram,
			args: Diagramming.SerializeTagEventArgs
		) => void;
		onDeserializeTag?: (
			sender: Diagramming.Diagram,
			args: Diagramming.SerializeTagEventArgs
		) => void;
		onTableColumnResizing?: (
			sender: Diagramming.Diagram,
			args: Diagramming.CellEventArgs
		) => void;
		onTableColumnResized?: (
			sender: Diagramming.Diagram,
			args: Diagramming.CellEventArgs
		) => void;
		onTableRowResizing?: (
			sender: Diagramming.Diagram,
			args: Diagramming.CellEventArgs
		) => void;
		onTableRowResized?: (
			sender: Diagramming.Diagram,
			args: Diagramming.CellEventArgs
		) => void;
		onNodePasted?: (
			sender: Diagramming.Diagram,
			args: Diagramming.NodeEventArgs
		) => void;
		onLinkPasted?: (
			sender: Diagramming.Diagram,
			args: Diagramming.LinkEventArgs
		) => void;
		onHeaderStartResizing?: (
			sender: Diagramming.Diagram,
			args: Diagramming.HeaderResizeEventArgs
		) => void;
		onHeaderResized?: (
			sender: Diagramming.Diagram,
			args: Diagramming.HeaderResizeEventArgs
		) => void;
		onItemAdded?: (
			sender: Diagramming.Diagram,
			args: Diagramming.ItemEventArgs
		) => void;
		onItemRemoved?: (
			sender: Diagramming.Diagram,
			args: Diagramming.ItemEventArgs
		) => void;
		onAnimatedLayoutCompleted?: (
			sender: Diagramming.Diagram,
			args: Diagramming.NodeEventArgs
		) => void;
		onHyperlinkClicked?: (
			sender: Diagramming.Diagram,
			args: Diagramming.NodeEventArgs
		) => void;
		onNodeDomCreated?: (
			sender: Diagramming.Diagram,
			args: Diagramming.NodeEventArgs
		) => void;
		onNodeTextEditing?: (
			sender: Diagramming.Diagram,
			args: Diagramming.NodeEventArgs
		) => void;
		onLinkTextEditing?: (
			sender: Diagramming.Diagram,
			args: Diagramming.LinkEventArgs
		) => void;
		onCellTextEditing?: (
			sender: Diagramming.Diagram,
			args: Diagramming.CellEventArgs
		) => void;
		onTreeItemTextEditing?: (
			sender: Diagramming.Diagram,
			args: Diagramming.TreeItemEventArgs
		) => void;
		onTreeItemTextEdited?: (
			sender: Diagramming.Diagram,
			args: Diagramming.TreeItemEventArgs
		) => void;
		onNodeSelecting?: (
			sender: Diagramming.Diagram,
			args: Diagramming.NodeEventArgs
		) => void;
		onLinkSelecting?: (
			sender: Diagramming.Diagram,
			args: Diagramming.LinkEventArgs
		) => void;
		onSelectionChanged?: (
			sender: Diagramming.Diagram,
			args: Diagramming.SelectionChangedEventArgs
		) => void;
		onDataLoaded?: (
			sender: Diagramming.Diagram
		) => void;
		onMouseWheel?: (
			sender: Diagramming.Diagram,
			args: Diagramming.DiagramEventArgs
		) => void;
	}

	export class Overview extends React.Component<OverviewProps, any> { }
	interface OverviewProps {
		id?: String;
		diagramView: DiagramView | null,
		enabled?: boolean;
		backColor?: string;
		dimColor?: string;
		scaleFactor?: number;
		scaleMode?: Diagramming.ScaleMode;
		allowZoom?: boolean;
		autoScrollAmount?: number;
		style?: Object;
	}

	export class NodeListView extends React.Component<NodeListViewProps, any> {
		/** Adds a node to the NodeListView.
		 * @param node The node to add.
		 * @param caption The caption to display for the node.
		*/
		addNode(node: Diagramming.DiagramNode, caption?: string): void;
	}
	interface NodeListViewProps {
		id?: String;
		enabled?: boolean;
		nodes?: Array<Diagramming.DiagramNode>,
		captions?: Array<string>,
		iconSize?: Drawing.Size;
		defaultNodeSize?: Drawing.Size;
		padding?: number;
		orientation?: Diagramming.Orientation;
		tooltipDelay?: number;
		shapeLibraryLocation?: string;
		style?: Object;
		onNodeSelected?: (
			sender: Diagramming.NodeListView,
			args: Diagramming.NodeEventArgs
		) => void;
	}

	export class ZoomControl extends React.Component<ZoomControlProps, any> { }
	interface ZoomControlProps {
		id?: String;
		diagramView: DiagramView | null,
		enabled?: boolean;
		cornerRadius?: number;
		padding?: number;
		snapToZoomStep?: boolean;
		tickPosition?: Controls.TickPosition;
		showLabel?: boolean;
		textColor?: string;
		shadowColor?: string;
		innerColor?: string;
		borderColor?: string;
		activeColor?: string;
		fill?: string;
		backColor?: string;
		scrollStep?: number;
		zoomStep?: number;
		maxZoomFactor?: number;
		minZoomFactor?: number;
		zoomFactor?: number;
		style?: Object;
	}

	interface Ruler<RulerProps = {}, RulerState = {}> extends React.Component<RulerProps, RulerState> { }
	export class Ruler { }
	interface RulerProps {
		id: String;
		unit?: Drawing.GraphicsUnit;
		enableGuides?: boolean;
		projectRotatedBounds?: boolean;
		negatedY?: boolean;
		negatedX?: boolean;
		verticalScaleVisible?: boolean;
		horizontalScaleVisible?: boolean;
		pointerColor?: string;
		textColor?: string;
		guideColor?: string;
		projectionColor?: string;
		backColor?: string;
		foreColor?: string;
		style?: Object;
	}
}