import React, { Component } from 'react';
import * as Controls from '@mindfusion/controls';
import * as Diagramming from '@mindfusion/diagramming';

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _inherits(subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function");
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      writable: true,
      configurable: true
    }
  });
  if (superClass) _setPrototypeOf(subClass, superClass);
}

function _getPrototypeOf(o) {
  _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {
    return o.__proto__ || Object.getPrototypeOf(o);
  };
  return _getPrototypeOf(o);
}

function _setPrototypeOf(o, p) {
  _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
    o.__proto__ = p;
    return o;
  };

  return _setPrototypeOf(o, p);
}

function _isNativeReflectConstruct() {
  if (typeof Reflect === "undefined" || !Reflect.construct) return false;
  if (Reflect.construct.sham) return false;
  if (typeof Proxy === "function") return true;

  try {
    Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {}));
    return true;
  } catch (e) {
    return false;
  }
}

function _assertThisInitialized(self) {
  if (self === void 0) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return self;
}

function _possibleConstructorReturn(self, call) {
  if (call && (typeof call === "object" || typeof call === "function")) {
    return call;
  } else if (call !== void 0) {
    throw new TypeError("Derived constructors may only return object or undefined");
  }

  return _assertThisInitialized(self);
}

function _createSuper(Derived) {
  var hasNativeReflectConstruct = _isNativeReflectConstruct();

  return function _createSuperInternal() {
    var Super = _getPrototypeOf(Derived),
        result;

    if (hasNativeReflectConstruct) {
      var NewTarget = _getPrototypeOf(this).constructor;

      result = Reflect.construct(Super, arguments, NewTarget);
    } else {
      result = Super.apply(this, arguments);
    }

    return _possibleConstructorReturn(this, result);
  };
}

function _superPropBase(object, property) {
  while (!Object.prototype.hasOwnProperty.call(object, property)) {
    object = _getPrototypeOf(object);
    if (object === null) break;
  }

  return object;
}

function _get(target, property, receiver) {
  if (typeof Reflect !== "undefined" && Reflect.get) {
    _get = Reflect.get;
  } else {
    _get = function _get(target, property, receiver) {
      var base = _superPropBase(target, property);

      if (!base) return;
      var desc = Object.getOwnPropertyDescriptor(base, property);

      if (desc.get) {
        return desc.get.call(receiver);
      }

      return desc.value;
    };
  }

  return _get(target, property, receiver || target);
}

var ControlWrapper = /*#__PURE__*/function (_Component) {
  _inherits(ControlWrapper, _Component);

  var _super = _createSuper(ControlWrapper);

  function ControlWrapper(props) {
    var _this;

    _classCallCheck(this, ControlWrapper);

    _this = _super.call(this, props);
    _this.element = /*#__PURE__*/React.createRef();
    return _this;
  }
  /**
  * Sets a control property from props.
  * @private
  */


  _createClass(ControlWrapper, [{
    key: "setProp",
    value: function setProp(propName, instance) {
      instance = instance || this.control;

      if (this.props[propName] !== undefined) {
        instance[propName] = this.props[propName];
      }
    }
    /**
    * Sets all control properties from props.
    * @private
    */

  }, {
    key: "setProps",
    value: function setProps(prevProps) {
      var _this2 = this;

      if (prevProps) {
        this.controlProps.forEach(function (p) {
          if (_this2.props[p] !== prevProps[p]) _this2.setProp(p);
        });
      } else {
        this.controlProps.forEach(function (p) {
          _this2.setProp(p);
        });
      }
    }
    /**
    * Sets a control event from props.
    * @private
    */

  }, {
    key: "setEvent",
    value: function setEvent(eventName, instance) {
      var _this3 = this;

      instance = instance || this.control;
      var handlerName = "on" + eventName[0].toUpperCase() + eventName.substr(1);

      if (this.props[handlerName] !== undefined) {
        instance.addEventListener(eventName, function (sender, args) {
          _this3.props[handlerName](sender, args);
        });
      }
    }
    /**
    * Gets the list of control properties, that can be set from JSX.
    * @private
    */

  }, {
    key: "controlProps",
    get: function get() {
      return [];
    }
    /**
    * Binds a diagramView to the control. 
    * @private
    */

  }, {
    key: "bindDiagramView",
    value: function bindDiagramView(diagramView) {
      if (diagramView) {
        var view = diagramView.current ? diagramView.current.control : diagramView.control;
        if (view) this.attachDiagramView(view);
      }
    }
  }, {
    key: "componentDidMount",
    value: function componentDidMount() {
      this.attachControl();

      if (this.props.diagramView) {
        this.bindDiagramView(this.props.diagramView);
      }
    }
  }, {
    key: "componentDidUpdate",
    value: function componentDidUpdate(prevProps, prevState) {
      if (this.props.diagramView) this.bindDiagramView(this.props.diagramView);
      this.setProps(prevProps);
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      // dispose
      if (this.control) this.control.dispose();
    }
  }, {
    key: "render",
    value: function render() {
      return /*#__PURE__*/React.createElement("div", null);
    }
  }]);

  return ControlWrapper;
}(Component);

var DiagramView = /*#__PURE__*/function (_ControlWrapper) {
  _inherits(DiagramView, _ControlWrapper);

  var _super = _createSuper(DiagramView);

  function DiagramView() {
    _classCallCheck(this, DiagramView);

    return _super.apply(this, arguments);
  }

  _createClass(DiagramView, [{
    key: "attachControl",
    value: function attachControl() {
      var _this = this;

      this.control = Diagramming.DiagramView.create(this.element.current); // set props

      this.setProps();
      this.setDiagramProps();
      var diagram = this.control.diagram;
      Object.getOwnPropertyNames(Diagramming.Events).forEach(function (e) {
        if (e === "length" || e === "name" || e === "prototype") ; else if (e === "zoomChanged" || e === "sizeChanged" || e == "mouseWheel" || e == "controlLoaded" || e == "createEditControl" || e == "enterInplaceEditMode" || e == "leaveInplaceEditMode") _this.setEvent(e);else _this.setEvent(e, diagram);
      }); // listen for changes

      var changedDelegate = function changedDelegate() {
        _this.raiseDiagramChanged();
      };

      diagram.addEventListener(Diagramming.Events.nodeCreated, changedDelegate);
      diagram.addEventListener(Diagramming.Events.nodeModified, changedDelegate);
      diagram.addEventListener(Diagramming.Events.nodeDeleted, changedDelegate);
      diagram.addEventListener(Diagramming.Events.linkCreated, changedDelegate);
      diagram.addEventListener(Diagramming.Events.linkModified, changedDelegate);
      diagram.addEventListener(Diagramming.Events.linkDeleted, changedDelegate);
      diagram.addEventListener(Diagramming.Events.nodeTextEdited, changedDelegate);
      diagram.addEventListener(Diagramming.Events.linkTextEdited, changedDelegate);
      diagram.addEventListener(Diagramming.Events.cellTextEdited, changedDelegate);
      diagram.addEventListener(Diagramming.Events.laneGridCellTextEdited, changedDelegate);
      diagram.addEventListener(Diagramming.Events.containerChildAdded, changedDelegate);
      diagram.addEventListener(Diagramming.Events.containerChildRemoved, changedDelegate);
    }
  }, {
    key: "controlProps",
    get: function get() {
      return ["diagram", "allowInplaceEdit", "behavior", "enabled", "zoomFactor", "multiTouchZoom", "multiTouchModify", "enableMultiTouch"];
    }
  }, {
    key: "diagramProps",
    get: function get() {
      return ["autoResize", "backBrush", "defaultShape", "linkHeadShapeSize", "linkShape", "roundedLinks", "routeLinks", "showAnchors", "showGrid"];
    }
  }, {
    key: "raiseDiagramChanged",
    value: function raiseDiagramChanged() {
      // raise the diagramChanged event
      if (this.props.onDiagramChanged) this.props.onDiagramChanged(this, Controls.EventArgs.Empty);
    }
    /**
    * Sets diagram properties from props.
    * @private
    */

  }, {
    key: "setDiagramProps",
    value: function setDiagramProps(prevProps) {
      var _this2 = this;

      if (prevProps) {
        this.diagramProps.forEach(function (p) {
          if (_this2.props[p] !== prevProps[p]) {
            _this2.setProp(p, _this2.control.diagram);
          }
        });
      } else {
        this.diagramProps.forEach(function (p) {
          _this2.setProp(p, _this2.control.diagram);
        });
      }
    }
  }, {
    key: "componentDidMount",
    value: function componentDidMount() {
      this.attachControl();
    }
  }, {
    key: "componentDidUpdate",
    value: function componentDidUpdate(prevProps, prevState) {
      this.setDiagramProps(prevProps);
      this.setProps(prevProps);
    }
  }, {
    key: "render",
    value: function render() {
      var defaultStyle = {
        overflow: "auto",
        height: "100%",
        width: "100%",
        textAlign: "left"
      };
      var style = Object.assign({}, defaultStyle, this.props.style);
      return /*#__PURE__*/React.createElement("div", {
        className: "mf-react-diagram",
        style: style
      }, /*#__PURE__*/React.createElement("canvas", {
        width: "200",
        height: "200",
        ref: this.element,
        id: this.props.id
      }));
    }
  }]);

  return DiagramView;
}(ControlWrapper);

var Overview = /*#__PURE__*/function (_ControlWrapper) {
  _inherits(Overview, _ControlWrapper);

  var _super = _createSuper(Overview);

  function Overview() {
    _classCallCheck(this, Overview);

    return _super.apply(this, arguments);
  }

  _createClass(Overview, [{
    key: "attachControl",
    value: function attachControl() {
      this.control = Diagramming.Overview.create(this.element.current);
      this.setProps();
    }
  }, {
    key: "attachDiagramView",
    value: function attachDiagramView(diagramView) {
      if (this.control.diagramView !== diagramView) this.control.diagramView = diagramView;
    }
  }, {
    key: "controlProps",
    get: function get() {
      return ["enabled", "backColor", "dimColor", "scaleFactor", "scaleMode", "allowZoom", "autoScrollAmount"];
    }
  }, {
    key: "render",
    value: function render() {
      var defaultStyle = {
        height: "200px",
        width: "200px"
      };
      var style = Object.assign({}, defaultStyle, this.props.style);
      return /*#__PURE__*/React.createElement("div", {
        className: "mf-react-overview",
        style: style
      }, /*#__PURE__*/React.createElement("canvas", {
        width: "200",
        height: "200",
        ref: this.element,
        id: this.props.id
      }));
    }
  }]);

  return Overview;
}(ControlWrapper);

var NodeListView = /*#__PURE__*/function (_ControlWrapper) {
  _inherits(NodeListView, _ControlWrapper);

  var _super = _createSuper(NodeListView);

  function NodeListView() {
    _classCallCheck(this, NodeListView);

    return _super.apply(this, arguments);
  }

  _createClass(NodeListView, [{
    key: "addNode",
    value:
    /**
    * Adds a node to the NodeListView.
    * @param {DiagramNode} node The node to add.
    * @param {String} [caption] The caption to display for the node.
    */
    function addNode(node, caption) {
      this.control.addNode(node, caption);
    }
  }, {
    key: "attachControl",
    value: function attachControl() {
      this.control = Diagramming.NodeListView.create(this.element.current);
      this.setProps();
      this.setEvent("nodeSelected");
      this.resetNodes();
    }
  }, {
    key: "controlProps",
    get: function get() {
      return ["enabled", "iconSize", "defaultNodeSize", "padding", "orientation", "tooltipDelay", "shapeLibraryLocation"];
    }
    /**
    * Resets the control nodes from values in props.
    * @private
    */

  }, {
    key: "resetNodes",
    value: function resetNodes() {
      this.control.clearAll();

      if (this.props.nodes !== undefined) {
        for (var i = 0; i < this.props.nodes.length; ++i) {
          var caption = this.props.captions ? this.props.captions.length > i ? this.props.captions[i] : "" : "";
          this.control.addNode(this.props.nodes[i], caption);
        }
      }
    }
  }, {
    key: "componentDidMount",
    value: function componentDidMount() {
      this.attachControl();
    }
  }, {
    key: "shouldComponentUpdate",
    value: function shouldComponentUpdate(nextProps) {
      return true;
    }
  }, {
    key: "componentDidUpdate",
    value: function componentDidUpdate(prevProps, prevState) {
      this.setProps(prevProps);

      if (prevProps.nodes !== this.props.nodes || prevProps.captions !== this.props.captions) {
        this.resetNodes();
      }
    }
  }, {
    key: "render",
    value: function render() {
      var defaultStyle = {
        height: "200px",
        width: "200px",
        overflowY: "auto",
        overflowX: "hidden"
      };
      var style = Object.assign({}, defaultStyle, this.props.style);
      return /*#__PURE__*/React.createElement("div", {
        className: "mf-react-nodelistview",
        style: style
      }, /*#__PURE__*/React.createElement("canvas", {
        width: "200",
        height: "200",
        ref: this.element,
        id: this.props.id
      }));
    }
  }]);

  return NodeListView;
}(ControlWrapper);

var ZoomControl = /*#__PURE__*/function (_ControlWrapper) {
  _inherits(ZoomControl, _ControlWrapper);

  var _super = _createSuper(ZoomControl);

  function ZoomControl() {
    _classCallCheck(this, ZoomControl);

    return _super.apply(this, arguments);
  }

  _createClass(ZoomControl, [{
    key: "attachControl",
    value: function attachControl() {
      this.control = Controls.ZoomControl.create(this.element.current);
      this.setProps();
    }
  }, {
    key: "attachDiagramView",
    value: function attachDiagramView(diagramView) {
      if (this.control.target !== diagramView) this.control.target = diagramView;
    }
  }, {
    key: "controlProps",
    get: function get() {
      return ["enabled", "cornerRadius", "padding", "snapToZoomStep", "tickPosition", "showLabel", "textColor", "shadowColor", "innerColor", "borderColor", "activeColor", "fill", "backColor", "scrollStep", "zoomStep", "maxZoomFactor", "minZoomFactor", "zoomFactor"];
    }
  }, {
    key: "render",
    value: function render() {
      var defaultStyle = {
        height: 300,
        width: 50
      };
      var style = Object.assign({}, defaultStyle, this.props.style);
      return /*#__PURE__*/React.createElement("div", {
        className: "mf-react-zoomcontrol",
        style: style
      }, /*#__PURE__*/React.createElement("canvas", {
        width: "50",
        height: "300",
        ref: this.element,
        id: this.props.id
      }));
    }
  }]);

  return ZoomControl;
}(ControlWrapper);

var Ruler = /*#__PURE__*/function (_ControlWrapper) {
  _inherits(Ruler, _ControlWrapper);

  var _super = _createSuper(Ruler);

  function Ruler(props) {
    var _this;

    _classCallCheck(this, Ruler);

    _this = _super.call(this, props);
    _this.diagramViewControl = /*#__PURE__*/React.createRef();
    return _this;
  }

  _createClass(Ruler, [{
    key: "attachControl",
    value: function attachControl() {
      this.control = Diagramming.Ruler.create(this.element.current);
      this.setProps();
    }
  }, {
    key: "attachDiagramView",
    value: function attachDiagramView(diagramView) {
      if (this.control.diagramView !== diagramView) this.control.diagramView = diagramView;
    }
  }, {
    key: "controlProps",
    get: function get() {
      return ["unit", "enableGuides", "projectRotatedBounds", "negatedY", "negatedX", "verticalScaleVisible", "horizontalScaleVisible", "pointerColor", "textColor", "guideColor", "projectionColor", "backColor", "foreColor"];
    }
  }, {
    key: "diagramViewChild",
    get: function get() {
      var canvas = this.element.current.querySelector("canvas");
      if (canvas) return Diagramming.DiagramView.find(canvas.id);
      return null;
    }
  }, {
    key: "componentDidMount",
    value: function componentDidMount() {
      this.attachControl();
      if (this.diagramViewChild) this.attachDiagramView(this.diagramViewChild);
    }
  }, {
    key: "componentDidUpdate",
    value: function componentDidUpdate(prevProps, prevState) {
      _get(_getPrototypeOf(Ruler.prototype), "componentDidUpdate", this).call(this, prevProps, prevState);

      this.control.refreshScales();
    }
  }, {
    key: "render",
    value: function render() {
      var defaultStyle = {
        display: "inline-block",
        textAlign: "left"
      };
      var style = Object.assign({}, defaultStyle, this.props.style);
      this.diagramViewControl = React.Children.only(this.props.children);
      return /*#__PURE__*/React.createElement("div", {
        className: "mf-react-ruler",
        ref: this.element,
        id: this.props.id,
        style: style
      }, this.diagramViewControl);
    }
  }]);

  return Ruler;
}(ControlWrapper);

export { DiagramView, NodeListView, Overview, Ruler, ZoomControl };
